package org.jboss.soa.qa.transactiontest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.switchyard.component.bean.Service;

import javax.naming.InitialContext;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

@Service(GTRollbackComponent.class)
public class GTRollbackComponentBean implements GTRollbackComponent {

	private static final Logger logger = LoggerFactory.getLogger(GTRollbackComponentBean.class);
	private static final String JNDI_TRANSACTION_MANAGER = "java:jboss/TransactionManager";

	private int rollbackCounter = 0;

	@Override
	public void processEjbTransactionTasksAndRollback(String command) {
		Transaction t = getCurrentTransaction();
		try {
			if (++rollbackCounter % 3 != 0) {
				try {
					Thread.sleep(500);
				} catch (InterruptedException e) {
					logger.error("Can not wait to process rollback.", e);
				}
				t.setRollbackOnly();
				logger.info("Processing rollback for transaction: {}", t.toString());
				logger.info("Marked transaction to rollback!");
			} else {
				logger.info("Rollbacks completed - will be committed");
			}
		} catch (Exception e) {
			logger.error("Failed to rollback transaction", e);
		}
	}

	private Transaction getCurrentTransaction() {
		try {
			TransactionManager tm = (TransactionManager) new InitialContext().lookup(JNDI_TRANSACTION_MANAGER);
			Transaction t =  tm.getTransaction();
			if (t == null) {
				logger.error("No active transaction");
			}
			return t;
		} catch (Exception e) {
			logger.error("Failed to get current transaction");
			return null;
		}
	}
}
