package org.jboss.soa.qa.transactiontest;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.importer.ZipImporter;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.soa.qa.transactiontest.model.JpaCommand;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import javax.annotation.Resource;
import javax.inject.Inject;
import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.MessageProducer;
import javax.jms.QueueConnectionFactory;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.transaction.UserTransaction;
import java.io.File;
import java.util.zip.ZipFile;

@RunWith(Arquillian.class)
public class BPMNSqlTransactionIssueTest {

	private static final String JAR_FILE = "target/bpmn-sql-transaction-issue.jar";

	@Deployment
	public static Archive<?> createTestArchive() throws Exception {
		File artifact = new File(JAR_FILE);
		try {
			return ShrinkWrap.create(ZipImporter.class, artifact.getName()).importFrom(new ZipFile(artifact)).as(JavaArchive.class);
		} catch (Exception e) {
			throw new RuntimeException(JAR_FILE + " not found. Run 'mvn package'", e);
		}
	}

	@PersistenceContext(unitName = "creatingPU")
	private EntityManager em;

	@Inject
	UserTransaction t;

	@Resource(mappedName = "java:/JmsXA")
	private QueueConnectionFactory connectionFactory;

	@Resource(mappedName = "qJcaIn")
	private Destination qJcaIn;

	@Resource(mappedName = "qCamelJmsIn")
	private Destination qCamelJmsIn;

	@Before
	public void cleanDB() throws Exception {
		t.begin();
		em.createQuery("DELETE FROM GGTSqlEntity e").executeUpdate();
		em.createQuery("DELETE FROM GLTSqlEntity e").executeUpdate();
		t.commit();
	}

	@Test
	public void testJPABindingTransaction() throws Exception {
		t.begin();
		JpaCommand jpaCommand = new JpaCommand();
		jpaCommand.setCommand("rollback Camel JPA binding");
		em.persist(jpaCommand);
		t.commit();

		verifyEntities();
	}

	@Test
	public void testJCABindingTransaction() throws Exception {
		sendJmsMessage(qJcaIn, "rollback JCA binding");
		verifyEntities();
	}

	@Test
	public void testJMSBindingTransaction() throws Exception {
		sendJmsMessage(qCamelJmsIn, "rollback Camel JMS binding");

		verifyEntities();
	}

	private void sendJmsMessage(Destination destination, String text) throws Exception {
		Connection connection = connectionFactory.createConnection();
		connection.start();

		try {
			Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
			MessageProducer producer = session.createProducer(destination);
			TextMessage message = session.createTextMessage();
			message.setText(text);
			producer.send(message);
			session.close();
		} finally {
			connection.close();
		}
	}

	private void verifyEntities() throws Exception {
		Thread.sleep(15000); // wait for rollbacks and final commit

		// DBEntity processed
		Assert.assertEquals(1, em.createQuery("SELECT e FROM GGTSqlEntity e").getResultList().size());
		Assert.assertEquals(3, em.createQuery("SELECT e FROM GLTSqlEntity e").getResultList().size());
	}
}
